"""GROM (Thunder) search space constants — code-enforced bounds only."""
from wisent.core.utils.config_tools.constants.for_experiments.by_method._tecza import *  # noqa: F401,F403

# --- GROM code-enforced bounds ---
SS_GROM_RETAIN_WEIGHT_LOW = 0.0
SS_GROM_RETAIN_WEIGHT_HIGH = 1.0
SS_GROM_ETA_MIN_FACTOR_LOW = 0.0
SS_GROM_ETA_MIN_FACTOR_HIGH = 1.0
SS_GROM_LINEAR_THRESH_LOW = 0.0
SS_GROM_LINEAR_THRESH_HIGH = 1.0
SS_GROM_ADAPT_CONE_LOW = 0.0
SS_GROM_ADAPT_CONE_HIGH = 1.0
SS_GROM_ADAPT_MANIFOLD_LOW = 0.0
SS_GROM_ADAPT_MANIFOLD_HIGH = 1.0
SS_GROM_ADAPT_LINEAR_DIR_MIN = 1
SS_GROM_ADAPT_LINEAR_DIR_MAX = 3
SS_GROM_ADAPT_COMPLEX_DIR_MIN = 2
SS_GROM_ADAPT_COMPLEX_DIR_MAX = 8
SS_GROM_ADAPT_MAX_DIR_MIN = 3
SS_GROM_ADAPT_MAX_DIR_MAX = 10
SS_GROM_SIG_DIR_DEFAULT_MIN = 1
SS_GROM_SIG_DIR_DEFAULT_MAX = 5
SS_GROM_MIN_ADAPTED_DIR_MIN = 1
SS_GROM_MIN_ADAPTED_DIR_MAX = 3
SS_GROM_CAA_SIM_SKIP_LOW = 0.0
SS_GROM_CAA_SIM_SKIP_HIGH = 1.0
SS_GROM_CONTRASTIVE_MARGIN_LOW = 0.0
SS_GROM_CONTRASTIVE_MARGIN_HIGH = 2.0
SS_GROM_MIN_COS_LOW = -1.0
SS_GROM_MIN_COS_HIGH = 1.0
SS_GROM_MAX_COS_LOW = -1.0
SS_GROM_MAX_COS_HIGH = 1.0
SS_GROM_GATE_THRESH_LOW = 0.0
SS_GROM_GATE_THRESH_HIGH = 1.0
SS_GROM_DIM_MIN = 8
SS_GROM_DIM_MAX = 128
SS_GROM_DIM_DIVISOR_MIN = 1
SS_GROM_DIM_DIVISOR_MAX = 8
SS_GROM_SHRINK_MIN = 1
SS_GROM_SHRINK_MAX = 4
SS_GROM_LOG_INTERVAL_MIN = 10
SS_GROM_LOG_INTERVAL_MAX = 100
SS_GROM_BEHAVIOR_TARGET = 1.0
